/*-----------------------------------------------------------------------------
 * Name:    ADC_S32K144-EVB.c
 * Purpose: A/D Converter interface for NXP S32K144-EVB evaluation board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2017 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "device_registers.h"           // Device header
#include "pcc_hal.h"                    // Keil::Device:SDK HAL:PCC
#include "port_hal.h"                   // Keil::Device:SDK HAL:PORT
#include "gpio_hal.h"                   // Keil::Device:SDK HAL:GPIO
#include "adc_hal.h"                    // Keil::Device:SDK HAL:ADC
#include "Board_ADC.h"                  /* ::Board Support:A/D Converter */

/* Board has a potentiometer that maps as follows:
     ADC0 channel 12 (PTC14)
*/

#define ADC_RESOLUTION        12U        /* Number of A/D converter bits */

static volatile uint16_t AD_last;       /* Last converted value */
static volatile uint8_t  AD_done;       /* AD conversion done flag */

/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {

  /* Enable Clock to Port C */
  PCC_HAL_SetClockMode(PCC, PCC_PORTC_CLOCK, 1U);

  if ((SCG->FIRCDIV & (7U << 8)) == 0U) {
    SCG->FIRCDIV |= (1U << 8);                                   /* ADC0 uses FIR as clock source therefore enable FIR clock output 2 */
  }

  PCC_HAL_SetClockMode(PCC, PCC_ADC0_CLOCK, 0U);                 /* disable ADC0 clock */
  PCC_HAL_SetClockSourceSel(PCC, PCC_ADC0_CLOCK, CLK_SRC_FIRC);  /* peripheral clock source = fast IRC clock */
  PCC_HAL_SetClockMode(PCC, PCC_ADC0_CLOCK, 1U);                 /* enable ADC0 clock */

  ADC_HAL_Init(ADC0);                                            /* Initializes the ADC0 to reset values */
  ADC_HAL_SetInputClock(ADC0, ADC_CLK_ALT_1);                    /* Bus clock */
  ADC_HAL_SetClockDivide(ADC0, ADC_CLK_DIVIDE_1);                /* input clock / 1 */
  ADC_HAL_SetResolution(ADC0, ADC_RESOLUTION_12BIT);             /* 12-bit conversion */

  ADC_HAL_SetSampleTime(ADC0, 12U);

  ADC_HAL_SetTriggerMode(ADC0, ADC_TRIGGER_SOFTWARE);            /* software trigger selected */
  ADC_HAL_SetVoltageReference(ADC0, ADC_VOLTAGEREF_VREF);        /* default voltage reference pin pair */

  ADC_HAL_SetContinuousConvFlag(ADC0, 0U);                       /* one conversion */
  ADC_HAL_SetHwAverageEnableFlag(ADC0, 1U);                      /* hardware average function enabled */
  ADC_HAL_SetHwAverageMode(ADC0, ADC_AVERAGE_32);                /* 32 samples averaged */

  NVIC_EnableIRQ(ADC0_IRQn);                                     /* enable ADC0 interrupt */

  return 0;
}

/**
  \fn          int32_t ADC_Uninitialize (void)
  \brief       De-initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Uninitialize (void) {

  NVIC_DisableIRQ(ADC0_IRQn);                                    /* disable ADC interrupt */
  PCC_HAL_SetClockMode(PCC, PCC_ADC0_CLOCK, 0U);                 /* disable ADC0 clock */

  return 0;
}

/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {

  ADC_HAL_SetChanInterruptEnableFlag(ADC0, 0U, 1U);              /* enable Conversion complete interrupt */
  ADC_HAL_SetInputChannel(ADC0, 0U, ADC_INPUTCHAN_AD12);         /* channel AD12 */

  return 0;
}

/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {
  return (AD_done ? 0 : -1);
}

/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {

  if (AD_done) {
    AD_done = 0;
    return AD_last;
  }
  return -1;
}

/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {
  return ADC_RESOLUTION;
}

/**
  \fn          void ADC_IRQHandler (void)
  \brief       Analog-to-Digital Converter Interrupt Handler
*/
void ADC0_IRQHandler(void) {
 uint16_t ADC_reg;

  ADC_reg = ADC_HAL_GetChanResult(ADC0, 0U);                     /* read conversion result */
  AD_last = ADC_reg;                                             /* store converted value */

  AD_done = 1;
}
