/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/* ###################################################################
**     Filename    : main.c
**     Project     : ammclib
**     Processor   : S32K144_100
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2017-02-07, 10:29, # CodeGen: 0
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/
/*!
**  @addtogroup main_module main module documentation
**  @{
*/
/* MODULE main */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "clockMan1.h"
#include "pin_mux.h"
#include "AMMCLib1.h"
#include "lpTmr1.h"
#include "flexTimer1.h"

#if CPU_INIT_CONFIG
  #include "Init_Config.h"
#endif

volatile int exit_code = 0;
/* User includes (#include below this line is not maintained by Processor Expert) */
#include <stdint.h>
#include <stdbool.h>
#include <math.h>

/* Defines for various PI usage */
#define PI          (3.14f)
#define TWO_PI      (2 * PI)
#define PI_DIV3     (PI / 3)
#define TWO_PI_DIV3 (TWO_PI / 3)

/* Sample frequency definition */
#define SAMPLE_FREQ      (10000.0f)
/* Macro that will convert a frequency to a sine argument */
#define SINE_PARAM(x)    (2 * PI * (x / SAMPLE_FREQ))
/* Macro that will scale the sine value to duty cycle */
#define SINE_TO_DUTYCYCLE(x) ((1UL + x) * (FTM_MAX_DUTY_CYCLE / 2))

/* Flag that signals if new data has been generated */
volatile bool newData;
/* Vector used to store sine samples for RGB colors */
volatile float RGB[3];
#define R (0U)
#define G (1U)
#define B (2U)

/**
 * @brief This function will convert angles in degrees in radians
 * @param angleInDegrees Angle to convert
 * @return The angle in radians, normalized to [-pi, pi)
 */
float DegreeToRadian(float angleInDegrees)
{
	/* Perform a normalization of the angle in degrees to
	 * 2PI and then subtract PI to obtain a value in
	 * [-PI, PI)
	 */
	return (fmodf(angleInDegrees, TWO_PI) - PI);
}

/**
 * @brief This function will generate a sine sample based on the
 *        input parameters
 * @param frequency Frequency in Hz. The value will be converted to
 * 		  a sine argument
 * @param amplitude The amplitude of the resulting sine signal
 * @param sample Sample number
 * @param phase Phase to be added to the sine argument
 * @return The value of the sine signal at the respective sample
 */
float GenerateSineSignal(float frequency, float amplitude, uint32_t sample, float phase)
{
	float normalizedAngle;
	float sineArgInDegrees;
	/* Perform a Multiply and Accumulate operation on the sine argument,
	 * sample number and phase.
	 * sineArgInDegrees = ((2 * pi * (f / fs) * sample) + phase)
	 */
	sineArgInDegrees = MLIB_Mac_FLT(phase, SINE_PARAM(frequency), (float)sample);
	/* Convert the angle to radians */
	normalizedAngle = DegreeToRadian(sineArgInDegrees);
	/* Calculate the sample sine value */
	return amplitude * GFLIB_Sin(normalizedAngle, GFLIB_SIN_DEFAULT);
}

/**
 * @brief LPTMR Interrupt Service Routine.
 * This ISR will generate the sine signals for the RGB Leds.
 */
void LPTMR_ISR(void)
{
	/* Sample number */
	static uint32_t sample = 0UL;
	/* Clear LPTMR interrupt flag */
	LPTMR_DRV_ClearCompareFlag(INST_LPTMR1);
	/* Generate one sine sample for each color. To create a gradient effect
	 * each sine will have a phase difference of PI/3.
	 */
	RGB[R] = GenerateSineSignal(25.0f, 1.0f, sample, 0);
	RGB[G] = GenerateSineSignal(25.0f, 1.0f, sample, PI_DIV3);
	RGB[B] = GenerateSineSignal(25.0f, 1.0f, sample, TWO_PI_DIV3);
	/* Increment sample number */
	sample++;
	/* Signal that new data is ready */
	newData = true;
}

/*!
  \brief The main function for the project.
  \details The startup initialization sequence is the following:
 * - __start (startup asm routine)
 * - __init_hardware()
 * - main()
 *   - PE_low_level_init()
 *     - Common_Init()
 *     - Peripherals_Init()
*/
int main(void)
{
  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                 /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/
  /* FTM state structure declaration */
  ftm_state_t ftmState;
  /* Initialize and configure clocks
   * 	-	see clock manager component for details
   */
  CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
						g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
  CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);

  /* Initialize pins
   *	-	See PinSettings component for more info
   */
  PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

  /* Initialize LPTMR as a periodic counter with 10 ms compare value */
  LPTMR_DRV_Init(INST_LPTMR1, &lpTmr1_config0, false);
  /* Install the corresponding ISR and enable the interrupt */
  INT_SYS_InstallHandler(LPTMR0_IRQn, LPTMR_ISR, (isr_t *)NULL);
  INT_SYS_EnableIRQ(LPTMR0_IRQn);

  /* Initialize FTM in PWM mode */
  FTM_DRV_Init(INST_FLEXTIMER1, &flexTimer1_InitConfig, &ftmState);
  /* Initialize PWM channels */
  FTM_DRV_InitPwm(INST_FLEXTIMER1, &flexTimer1_PwmConfig);

  /* Clear newData flag */
  newData = false;
  /* Start LPTMR counter */
  LPTMR_DRV_StartCounter(INST_LPTMR1);

  /* Infinite loop */
  while(1)
  {
	  /* When new data is ready, update FTM channels */
	  if(newData)
	  {
		  FTM_DRV_UpdatePwmChannel(INST_FLEXTIMER1, 0U, FTM_PWM_UPDATE_IN_DUTY_CYCLE, (uint16_t)SINE_TO_DUTYCYCLE(RGB[R]),0U, false);
		  FTM_DRV_UpdatePwmChannel(INST_FLEXTIMER1, 1U, FTM_PWM_UPDATE_IN_DUTY_CYCLE, (uint16_t)SINE_TO_DUTYCYCLE(RGB[G]),0U, false);
		  FTM_DRV_UpdatePwmChannel(INST_FLEXTIMER1, 2U, FTM_PWM_UPDATE_IN_DUTY_CYCLE, (uint16_t)SINE_TO_DUTYCYCLE(RGB[B]),0U, true);
		  newData = false;
	  }
  }

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the Freescale S32K series of microcontrollers.
**
** ###################################################################
*/
