/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "edmaTransfer.h"
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

/* configuration structures for simple loop transfers */
edma_loop_transfer_config_t loopConfig = {
    .majorLoopIterationCount = MAJOR_LOOP_COUNT,
    .srcOffsetEnable = false,
    .dstOffsetEnable = false,
    .minorLoopOffset = 0,
    .minorLoopChnLinkEnable = false,
    .minorLoopChnLinkNumber = 0,
    .majorLoopChnLinkEnable = false,
    .majorLoopChnLinkNumber = 0
};

edma_transfer_config_t transferConfig = {
    .srcAddr = 0,
    .destAddr = 0,
    .srcTransferSize = EDMA_TRANSFER_SIZE_1B,
    .destTransferSize = EDMA_TRANSFER_SIZE_1B,
    .srcOffset = 1,
    .destOffset = 1,
    .srcLastAddrAdjust = 0,
    .destLastAddrAdjust = 0,
    .srcModulo = EDMA_MODULO_OFF,
    .destModulo = EDMA_MODULO_OFF,
    .minorByteTransferCount = 0,
    .scatterGatherEnable = false,
    .scatterGatherNextDescAddr = 0,
    .interruptEnable = true,
    .loopTransferConfig = &loopConfig
};

/* Special callback for peripheral-to-memory transfers */
static void uartRxDmaCallback(void *parameter)
{
    /* retrieve the channel state */
    uint32_t channel = ((uint32_t)parameter);

    /* get the address of last byte transferred from the TCD */
    uint8_t * lastByte = (uint8_t *)(*(uint32_t *)(((uint32_t)EDMA_REG_BASE_ADDR) + 0x1010 + 0x20 * channel));
    lastByte--;

    /* set major iteration count to 0 (disable channel for future requests)
     * if the last byte in the string has been received
     * */
    if(*lastByte == '\n')
        EDMA_HAL_TCDSetMajorCount(EDMA_REG_BASE_ADDR, channel, 0);
}

/* This function triggers a single block memory-to-memory transfer. */
void triggerSingleBlock(uint8_t channel, uint8_t * srcBuff, uint8_t * dstBuff, uint32_t size)
{
    /* configure eDMA channel for a single block mem-to-mem transfer */
    EDMA_DRV_ConfigSingleBlockTransfer(channel, EDMA_TRANSFER_MEM2MEM, (uint32_t)srcBuff,
                                           (uint32_t)dstBuff, EDMA_TRANSFER_SIZE_1B, size);

    /* trigger a sw DMA request for this channel */
    EDMA_HAL_TriggerChannelStart(EDMA_REG_BASE_ADDR, channel);

    /* wait for the channel to complete */
    while(EDMA_HAL_TCDGetCurrentMajorCount(EDMA_REG_BASE_ADDR, channel) > 0)
        __asm("nop");
}

/* This function triggers a loop memory-to-memory transfer. */
void triggerLoopTransfer(uint8_t channel, uint8_t * srcBuff, uint8_t * dstBuff, uint32_t size)
{
    /* configure transfer source and destination addresses */
    transferConfig.srcAddr = (uint32_t)srcBuff;
    transferConfig.destAddr = (uint32_t)dstBuff;
    transferConfig.minorByteTransferCount = size / MAJOR_LOOP_COUNT;

    /* configure the eDMA channel for a loop transfer (via transfer configuration structure */
    EDMA_DRV_ConfigLoopTransfer(channel, &transferConfig);

    /* while major loop is not exhausted */
    while(EDMA_HAL_TCDGetCurrentMajorCount(EDMA_REG_BASE_ADDR, channel) > 0)
    {
        /* trigger sw DMA requests for this channel */
        EDMA_HAL_TriggerChannelStart(EDMA_REG_BASE_ADDR, channel);

        /* wait for the channel to complete */
        while(EDMA_HAL_TCDGetChannelActiveStatus(EDMA_REG_BASE_ADDR, channel))
            __asm("nop");
    }
}

/* This function triggers a scatter/gather memory-to-memory transfer. */
void triggerScatterGather(uint8_t channel, uint8_t * srcBuff, uint8_t * dstBuff, uint32_t size)
{
    uint8_t i;
    /* compute the transfer size for the scatter/gather transfers */
    uint32_t transferSize = size / SG_TCD_COUNT;

    /* no callback is needed for this feature, as the TCD will be
     * updated automatically by the eDMA engine
     */
    EDMA_DRV_InstallCallback(channel, NULL, NULL);

    /* configure the list of scatter/gather configuration structures */
    for(i = 0U; i < SG_TCD_COUNT; i++)
    {
        srcList[i].address = (uint32_t)srcBuff + i * transferSize;
        dstList[i].address = (uint32_t)dstBuff + i * transferSize;
        srcList[i].length = dstList[i].length = transferSize;
        srcList[i].type = dstList[i].type = EDMA_TRANSFER_MEM2MEM;
    }

    /* allocate a list of software TCD structures and pass it to the driver;
     * this buffer needs to contain 32 extra bytes, as the TCD structures must be 32-bytes aligned;
     */
    edma_software_tcd_t stcd[5];

    /* configure the channel for a scatter/gather list of transfers */
    EDMA_DRV_ConfigScatterGatherTransfer(channel, stcd, EDMA_TRANSFER_SIZE_2B, 12U, srcList, dstList, 4U);

    /* for each TCD in the scatter/gather list */
    for(i = 0U; i < 4U; i++)
    {
        /* trigger sw DMA requests for this channel */
        EDMA_HAL_TriggerChannelStart(EDMA_REG_BASE_ADDR, channel);

        /* wait for the channel to complete */
        while(EDMA_HAL_TCDGetChannelActiveStatus(EDMA_REG_BASE_ADDR,  channel))
            __asm("nop");
    }

    /* re-install the callback */
    EDMA_DRV_InstallCallback(channel, (edma_callback_t)edmaCallback, (void *)((uint32_t)channel));
}

/* This function triggers a memory-to-peripheral transfer. */
void triggerDmaUartTx(uint8_t channel, uint8_t * buff, uint32_t size)
{
    /* configure eDMA channel 0 for a single block memory-to-peripheral transfer */
    EDMA_DRV_ConfigSingleBlockTransfer(channel, EDMA_TRANSFER_MEM2PERIPH, (uint32_t)buff,
                                       (uint32_t)(&(LPUART1->DATA)), EDMA_TRANSFER_SIZE_1B, 1U);

    /* set the major loop count equal to the number of bytes to be transmitted;
     * each byte will be transferred during one DMA request */
    EDMA_HAL_TCDSetMajorCount(EDMA_REG_BASE_ADDR, channel, size);

    /* start the eDMA channel */
    EDMA_DRV_StartChannel(channel);

    /* enable DMA requests on data transmit */
    LPUART_HAL_SetTxDmaCmd(LPUART1, true);

    /* wait for the channels to complete */
    while(EDMA_HAL_TCDGetCurrentMajorCount(EDMA_REG_BASE_ADDR, channel) > 0)
        __asm("nop");

    /* stop the eDMA channel */
    EDMA_DRV_StopChannel(channel);

    /* disable DMA requests on data transmit */
    LPUART_HAL_SetTxDmaCmd(LPUART1, false);
}

/* This function triggers a peripheral-to-memory transfer. */
void triggerDmaUartRx(uint8_t channel, uint8_t * buff)
{
    /* configure eDMA channel 0 for a single block memory-to-peripheral transfer */
    EDMA_DRV_ConfigSingleBlockTransfer(channel, EDMA_TRANSFER_PERIPH2MEM, (uint32_t)(&(LPUART1->DATA)),
                                       (uint32_t)buff, EDMA_TRANSFER_SIZE_1B, 1U);

    /* special callback for this use-case; resets the major iteration count
     * only when '\n' character has been received
     */
    EDMA_DRV_InstallCallback(channel, (edma_callback_t)uartRxDmaCallback, (void *)((uint32_t)channel));

    /* start the channels */
    EDMA_DRV_StartChannel(channel);

    /* enable DMA requests on data receive */
    LPUART_HAL_SetRxDmaCmd(LPUART1, true);

    /* wait for the channels to complete */
    while(EDMA_HAL_TCDGetCurrentMajorCount(EDMA_REG_BASE_ADDR, channel) > 0)
        __asm("nop");

    /* stop the eDMA channel */
    EDMA_DRV_StopChannel(channel);

    /* disable DMA requests on data receive */
    LPUART_HAL_SetRxDmaCmd(LPUART1, false);

    /* re-install the initial callback */
    EDMA_DRV_InstallCallback(channel, (edma_callback_t)edmaCallback, (void *)((uint32_t)channel));
}
