/*
 * Copyright (c) 2015 - 2016 , Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NXP "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NXP OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/* ###################################################################
**     Filename    : main.c
**     Project     : power_mode_switch
**     Processor   : S32K144_100
**     Version     : Driver 01.00
**     Compiler    : GNU C Compiler
**     Date/Time   : 2016-11-15, 11:21, # CodeGen: 0
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.00
** @brief
**         Main module.
**         This module contains user's application code.
*/
/*!
**  @addtogroup main_module main module documentation
**  @{
*/
/* MODULE main */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "clockMan1.h"
#include "pin_mux.h"
#include "pwrMan1.h"
#include "lpuart1.h"
#include "dmaController1.h"
#include "osif1.h"
#if CPU_INIT_CONFIG
  #include "Init_Config.h"
#endif

volatile int exit_code = 0;
/* User includes (#include below this line is not maintained by Processor Expert) */

#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <stdio.h>

#define MENU_MESSAGE "Press:\r\n\
\t1) for HSRUN\r\n\
\t2) for RUN\r\n\
\t3) for VLPR\r\n\
\t4) for STOP\r\n\
\t5) for STOP1\r\n\
\t6) for STOP2\r\n\
\t7) for VLPS\r\n\
-->To wake up the CPU from stop modes press SW3\r\n\
Enter your input:"

#define SEPARATOR "---------------------------------------\r\n"

#define LED_PORT  (PTD)
#define LED_RED   (15u)
#define LED_GREEN (16u)

/* Definition of power modes indexes, as configured in Power Manager Component
 *  Refer to the Reference Manual for details about the power modes.
 */
#define HSRUN (0u) /* High speed run      */
#define RUN   (1u) /* Run                 */
#define VLPR  (2u) /* Very low power run  */
#define STOP  (3u) /* Stop mode           */
#define STOP1 (4u) /* Stop option 1       */
#define STOP2 (5u) /* Stop option 2       */
#define VLPS  (6u) /* Very low power stop */

#define POWER_MODES (7u)

/* brief: Send a string to user via LPUART
 * param sourceStr: pointer to the array of characters
 *                  that you wish to send.
 * return:          None
 */
void print(const char *sourceStr)
{
    uint32_t bytesRemaining;

    /* Send data via LPUART */
    LPUART_DRV_SendData(INST_LPUART1, (uint8_t *) sourceStr, strlen(sourceStr));
    /* Wait for transmission to be successful */
    while (LPUART_DRV_GetTransmitStatus(INST_LPUART1, &bytesRemaining) != STATUS_SUCCESS)
    {
    }
}

/* Function that reads the option chosen by the user */
void readOptionFromConsole(uint8_t * option)
{
    uint32_t bytesRemaining;
    bool strReceived = false;
    uint8_t i = 0;
    uint8_t buffer[255];

    while(strReceived == false)
    {
        /* Because the terminal appends new line to user data,
         * receive and store data into a buffer until it is received
         */
        LPUART_DRV_ReceiveData(INST_LPUART1, &buffer[i], 1UL);
        /* Wait for transfer to be completed */
        while(LPUART_DRV_GetReceiveStatus(INST_LPUART1, &bytesRemaining) != STATUS_SUCCESS);
        /* Check if current byte is new line */
        if((buffer[i++] == '\n') || (buffer[i] == '\r'))
            strReceived = true;
    }

    /* Check for invalid options */
    if(i > 2 || buffer[0] < '1' || buffer[0] > '7')
    {
        print((const char *)"Invalid option!\r\n");
        *option = 0;
    }
    else
    {
        *option = buffer[0];
    }
}

/* Interrupt service routine for SW3 */
void ButtonISR(void)
{
    /* Clear button IRQ flag */
    PORT_HAL_ClearPortIntFlagCmd(PORTC);
    /* Red off */
    GPIO_HAL_WritePin(LED_PORT, LED_RED, 1);
    /* Green on */
    GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 0);
}

/*!
  \brief The main function for the project.
  \details The startup initialization sequence is the following:
 * - __start (startup asm routine)
 * - __init_hardware()
 * - main()
 *   - PE_low_level_init()
 *     - Common_Init()
 *     - Peripherals_Init()
*/
int main(void)
{
    /* Variable used for storing user option */
    uint8_t option;
    /* Variable used for storing core frequency */
    uint32_t frequency;
    /* Buffer for sprintf operations */
    char buffer[20];

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  #ifdef PEX_RTOS_INIT
    PEX_RTOS_INIT();                 /* Initialization of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of Processor Expert internal initialization.                    ***/

    /* Initialize and configure clocks
     *  -   see clock manager component for details
     */
    CLOCK_SYS_Init(g_clockManConfigsArr, CLOCK_MANAGER_CONFIG_CNT,
                   g_clockManCallbacksArr, CLOCK_MANAGER_CALLBACK_CNT);
    CLOCK_SYS_UpdateConfiguration(0U, CLOCK_MANAGER_POLICY_AGREEMENT);

    /* Initialize pins
     *  -   See PinSettings component for more info
     */
    PINS_DRV_Init(NUM_OF_CONFIGURED_PINS, g_pin_mux_InitConfigArr);

    /* Initialize Power Manager */
    POWER_SYS_Init(&powerConfigsArr, POWER_MODES, &powerStaticCallbacksConfigsArr, 0);

    /* Initialize LPUART for host - board configurations */
    LPUART_DRV_Init(INST_LPUART1, &lpuart1_State, &lpuart1_InitConfig0);

    /* Install hander for Button ISR */
    INT_SYS_InstallHandler(PORTC_IRQn, ButtonISR, (isr_t *)0);
    /* Enable button port IRQ */
    INT_SYS_EnableIRQ(PORTC_IRQn);

    /* Red led off */
    GPIO_HAL_WritePin(LED_PORT, LED_RED, 1);
    /* Green led on */
    GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 0);

    while(1)
    {
        /* Print menu */
        print((const char *)MENU_MESSAGE);
        /* Read user option */
        readOptionFromConsole(&option);

        /* If the option is valid configure the CPU power mode accordingly */
        switch (option)
        {
        /* HSRUN */
        case '1':
            /* Set power mode to HSRUN */
            POWER_SYS_SetMode(HSRUN, POWER_MANAGER_POLICY_AGREEMENT);
            print((const char*)"******** CPU is in HSRUN\r\n");
            /* Get current core clock frequency */
            (void)CLOCK_SYS_GetFreq(CORE_CLOCK, &frequency);
            /* Transform the frequency from decimal to string */
            sprintf(buffer, "%ld", frequency);
            /* Print CPU frequency */
            print((const char*)"******** Core frequency: ");
            print(buffer);
            print((const char*)"[Hz] \r\n");
            break;
        /* RUN */
        case '2':
            /* Set power mode to RUN */
            POWER_SYS_SetMode(RUN, POWER_MANAGER_POLICY_AGREEMENT);
            print((const char*)"******** CPU is in RUN\r\n");
            /* Get current core clock frequency */
            (void)CLOCK_SYS_GetFreq(CORE_CLOCK, &frequency);
            /* Transform the frequency from decimal to string */
            sprintf(buffer, "%ld", frequency);
            /* Print CPU frequency */
            print((const char*)"******** Core frequency: ");
            print(buffer);
            print((const char*)"[Hz] \r\n");
            break;
        /* VLPR */
        case '3':
            /* Set power mode to VLPR */
            POWER_SYS_SetMode(VLPR, POWER_MANAGER_POLICY_AGREEMENT);
            print((const char*)"******** CPU is in VLPR\r\n");
            /* Get current core clock frequency */
            (void)CLOCK_SYS_GetFreq(CORE_CLOCK, &frequency);
            /* Transform the frequency from decimal to string */
            sprintf(buffer, "%ld", frequency);
            /* Print CPU frequency */
            print((const char*)"******** Core frequency: ");
            print(buffer);
            print((const char*)"[Hz] \r\n");
            break;
        /* STOP */
        case '4':
            print((const char*)"******** CPU is going in STOP...\r\n");
            /* Green off */
            GPIO_HAL_WritePin(LED_PORT, LED_RED, 0);
            /* Red on */
            GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 1);
            /* Set power mode to STOP */
            POWER_SYS_SetMode(STOP, POWER_MANAGER_POLICY_AGREEMENT);
            break;
        /* STOP1 */
        case '5':
            print((const char*)"******** CPU is going in STOP1...\r\n");
            /* Green off */
            GPIO_HAL_WritePin(LED_PORT, LED_RED, 0);
            /* Red on */
            GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 1);
            /* Set power mode to STOP1 */
            POWER_SYS_SetMode(STOP1, POWER_MANAGER_POLICY_AGREEMENT);
            break;
        /* STOP2 */
        case '6':
            print((const char*)"******** CPU is going in STOP2...\r\n");
            /* Green off */
            GPIO_HAL_WritePin(LED_PORT, LED_RED, 0);
            /* Red on */
            GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 1);
            /* Set power mode to STOP2 */
            POWER_SYS_SetMode(STOP2, POWER_MANAGER_POLICY_AGREEMENT);
            break;
        /* VLPS */
        case '7':
            print((const char*)"******** CPU is going in VLPS...\r\n");
            /* Green off */
            GPIO_HAL_WritePin(LED_PORT, LED_RED, 0);
            /* Red on */
            GPIO_HAL_WritePin(LED_PORT, LED_GREEN, 1);
            /* Set power mode to VLPS */
            POWER_SYS_SetMode(VLPS, POWER_MANAGER_POLICY_AGREEMENT);
            break;
        default:
            /* This statement should not be reached */
            break;
        }

        /* Print separator */
        print((const char *)SEPARATOR);
    }


  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;) {
    if(exit_code != 0) {
      break;
    }
  }
  return exit_code;
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.1 [05.21]
**     for the Freescale S32K series of microcontrollers.
**
** ###################################################################
*/

